import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.secret.connector.aws_secret_manager_connector import AWSSecretManagerConnector
from spaceone.secret.connector.vault_connector import VaultConnector
from spaceone.secret.connector.consul_connector import ConsulConnector
from spaceone.secret.error import *

_LOGGER = logging.getLogger(__name__)


class SecretConnectorManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        connector = config.get_global('CONNECTORS')
        backend = config.get_global('BACKEND', 'AWSSecretManagerConnector')
        try:
            _LOGGER.debug(f'[SecretConnectorManager] Create {backend}')
            self.secret_conn = self.locator.get_connector(backend)
        except Exception as e:
            _LOGGER.error(f'[SecretConnectorManager] not defined backend {backend}')
            raise ERROR_DEFINE_SECRET_BACKEND(backend=backend)

    def create_secret(self, secret_id, data):
        def _rollback(secret_id):
            _LOGGER.info(f'[ROLLBACK] Delete secret data in secret store : {secret_id}')
            self.secret_conn.delete_secret(secret_id)

        response = self.secret_conn.create_secret(secret_id, data)
        self.transaction.add_rollback(_rollback, secret_id)

    def update_secret(self, secret_id, data):
        self.secret_conn.update_secret(secret_id, data)

    def delete_secret(self, secret_id):
        self.secret_conn.delete_secret(secret_id)

    def get_secret(self, secret_id):
        return self.secret_conn.get_secret(secret_id)



from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

class IdentityManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.identity_conn: SpaceConnector = self.locator.get_connector('SpaceConnector', service='identity')

    def get_service_account(self, service_account_id, domain_id):
        return self.identity_conn.dispatch(
            'ServiceAccount.get', {'service_account_id': service_account_id, 'domain_id': domain_id})

    def list_service_accounts(self, query, domain_id):
        return self.identity_conn.dispatch(
            'ServiceAccount.list', {'query': query, 'domain_id': domain_id})

    def get_project(self, project_id, domain_id):
        return self.identity_conn.dispatch(
            'Project.get', {'project_id': project_id, 'domain_id': domain_id})

    def list_projects(self, query, domain_id):
        return self.identity_conn.dispatch(
            'Project.list', {'query': query, 'domain_id': domain_id})



import logging
import functools
from spaceone.api.secret.v1 import secret_group_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.secret.model.secret_group_model import SecretGroup
from spaceone.secret.info.secret_info import SecretInfo

__all__ = ['SecretGroupSecretInfo', 'SecretGroupInfo', 'SecretGroupsInfo']
_LOGGER = logging.getLogger(__name__)


def SecretGroupSecretInfo(secret_group_map_vo):
    info = {
        'secret_group_info': SecretGroupInfo(secret_group_map_vo.secret_group, minimal=True),
        'secret_info': SecretInfo(secret_group_map_vo.secret, minimal=True),
        'domain_id': secret_group_map_vo.secret.domain_id
    }

    return secret_group_pb2.SecretGroupSecretInfo(**info)


def SecretGroupInfo(secret_group_vo: SecretGroup, minimal=False):
    info = {
        'secret_group_id': secret_group_vo.secret_group_id,
        'name': secret_group_vo.name
    }

    if minimal is False:
        info.update({
            'tags': change_struct_type(secret_group_vo.tags),
            'domain_id': secret_group_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(secret_group_vo.created_at)
        })

    return secret_group_pb2.SecretGroupInfo(**info)


def SecretGroupsInfo(secret_group_vos, total_count, **kwargs):
    results = list(map(functools.partial(SecretGroupInfo, **kwargs), secret_group_vos))

    return secret_group_pb2.SecretGroupsInfo(results=results, total_count=total_count)



import logging

from spaceone.core.service import *
from spaceone.secret.error.custom import *
from spaceone.secret.manager.identity_manager import IdentityManager
from spaceone.secret.manager.secret_manager import SecretManager
from spaceone.secret.model.secret_model import Secret
from spaceone.secret.manager.trusted_secret_manager import TrustedSecretManager
from spaceone.secret.manager.secret_connector_manager import SecretConnectorManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler(exclude=['get_data'])
@mutation_handler
@event_handler
class SecretService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_mgr: SecretManager = self.locator.get_manager('SecretManager')

    @transaction(append_meta={
        'authorization.scope': 'PROJECT',
        'authorization.require_project_id': True
    })
    @check_required(['name', 'data', 'secret_type', 'domain_id'])
    def create(self, params):
        """ Create secret

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'secret_type': 'str',
                'tags': 'dict',
                'schema': 'str',
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'service_account_id': 'str',
                'project_id': 'str',
                'trusted_secret_id': 'str',
                'domain_id': 'str
            }

        Returns:
            secret_vo
        """

        domain_id = params['domain_id']

        if 'service_account_id' in params:
            service_account_info = self._get_service_account(params['service_account_id'], domain_id)
            params['provider'] = service_account_info.get('provider')
            params['project_id'] = service_account_info.get('project_info', {}).get('project_id')
        else:
            if 'project_id' in params:
                self._check_project(params['project_id'], domain_id)

        if 'trusted_secret_id' in params:
            trusted_secret_mgr = self.locator.get_manager('TrustedSecretManager')
            trusted_secret_mgr.get_trusted_secret(params['trusted_secret_id'], domain_id)

        secret_vo = self.secret_mgr.create_secret(params)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        secret_conn_mgr.create_secret(secret_vo.secret_id, params['data'])

        return secret_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['secret_id', 'domain_id'])
    def update(self, params):
        """ Update secret

        Args:
            params (dict): {
                'secret_id': 'str',
                'name': 'str' ,
                'tags': 'dict',
                'project_id': 'str',
                'release_project': 'bool',
                'domain_id': 'str'
            }

        Returns:
            secret_vo
        """

        domain_id = params['domain_id']
        secret_id = params['secret_id']
        project_id = params.get('project_id')
        release_project = params.get('release_project', False)

        secret_vo = self.secret_mgr.get_secret(secret_id, domain_id)

        if release_project:
            params['project_id'] = None
        else:
            if project_id:
                self._check_project(project_id, domain_id)

        secret_vo = self.secret_mgr.update_secret_by_vo(params, secret_vo)

        return secret_vo

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['secret_id', 'domain_id'])
    def delete(self, params):
        """ Delete secret

        Args:
            params (dict): {
                'secret_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        domain_id = params['domain_id']
        secret_id = params['secret_id']

        secret_vo = self.secret_mgr.get_secret(secret_id, domain_id)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        secret_conn_mgr.delete_secret(secret_id)

        self.secret_mgr.delete_secret_by_vo(secret_vo)

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['secret_id', 'data', 'domain_id'])
    def update_data(self, params):
        """ Update secret data through backend Secret service

        Args:
            params (dict): {
                'secret_id': 'str',
                'data': 'dict',
                'schema': 'str',
                'encrypted': 'bool',
                'encrypt_options': 'dict',
                'domain_id': 'str'
            }

        Returns:
            secret_data (dict)
        """
        domain_id = params['domain_id']
        secret_id = params['secret_id']
        data = params['data']

        secret_vo = self.secret_mgr.get_secret(secret_id, domain_id)
        self.secret_mgr.update_secret_by_vo(params, secret_vo)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        secret_conn_mgr.update_secret(secret_id, data)

    @transaction(append_meta={'authorization.scope': 'SYSTEM'})
    @check_required(['secret_id', 'domain_id'])
    def get_data(self, params):
        """ Get secret data through backend Secret service

        Args:
            params (dict): {
                'secret_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            secret_data (dict)
        """

        secret_id = params['secret_id']
        domain_id = params['domain_id']

        secret_vo: Secret = self.secret_mgr.get_secret(secret_id, domain_id)
        secret_data = self._get_secret_data(secret_id)
        encrypt_options = secret_vo.encrypt_options

        if secret_vo.trusted_secret_id:
            trusted_secret_mgr: TrustedSecretManager = self.locator.get_manager('TrustedSecretManager')
            trusted_secret_vo = trusted_secret_mgr.get_trusted_secret(trusted_secret_id=secret_vo.trusted_secret_id,
                                                                      domain_id=domain_id)

            self._check_validation_trusted_secret(secret_vo, trusted_secret_vo)

            trusted_secret_data = self._get_secret_data(trusted_secret_vo.trusted_secret_id)
            trusted_secret_encrypt_options = trusted_secret_vo.encrypt_options

            if secret_vo.encrypted and trusted_secret_vo.encrypted:
                secret_data['trusted_encrypted_data'] = trusted_secret_data['encrypted_data']

                encrypt_options.update({
                    'trusted_encrypted_data_key': trusted_secret_encrypt_options.get('encrypted_data_key')
                })
            elif secret_vo.encrypted is False and trusted_secret_vo.encrypted is False:
                # Merge secret data & trusted secret data
                trusted_secret_data.update(secret_data)
                secret_data = trusted_secret_data

        return {
            'encrypted': secret_vo.encrypted,
            'encrypt_options': encrypt_options,
            'data': secret_data
        }

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['secret_id', 'domain_id'])
    def get(self, params):
        """ Get secret

        Args:
            params (dict): {
                'secret_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            secret_vo
        """

        return self.secret_mgr.get_secret(params['secret_id'], params['domain_id'], params.get('only'))

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['domain_id'])
    @append_query_filter(['secret_id', 'name', 'secret_type', 'schema', 'provider',
                          'service_account_id', 'trusted_secret_id', 'domain_id', 'user_projects'])
    @append_keyword_filter(['secret_id', 'name', 'schema', 'provider'])
    def list(self, params):
        """ List secrets

        Args:
            params (dict): {
                'secret_id': 'str',
                'name': 'str',
                'secret_type': 'str',
                'schema': 'str',
                'provider': 'str',
                'service_account_id': 'str',
                'trusted_secret_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)',
                'user_projects': 'list', // from meta
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get('query', {})
        include_secret_group = params.get('include_secret_group', False)
        secret_vos, total_count = self.secret_mgr.list_secrets(query)

        if include_secret_group:
            secret_vos = self.secret_mgr.get_related_secret_groups(secret_vos)

        return secret_vos, total_count

    @transaction(append_meta={'authorization.scope': 'PROJECT'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['secret_id', 'name', 'schema', 'provider'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.secret_mgr.stat_secrets(query)

    def _get_secret_data(self, secret_id):
        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        return secret_conn_mgr.get_secret(secret_id)

    def _get_service_account(self, service_account_id, domain_id):
        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        return identity_mgr.get_service_account(service_account_id, domain_id)

    def _check_project(self, project_id, domain_id):
        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        identity_mgr.get_project(project_id, domain_id)

        return True

    @staticmethod
    def _check_validation_trusted_secret(secret_vo, trusted_secret_vo):
        if secret_vo.encrypted != trusted_secret_vo.encrypted:
            raise ERROR_DIFF_SECRET_AND_TRUSTED_SECRET_ENCRYPTED()

        if secret_vo.encrypted and trusted_secret_vo.encrypted:
            secret_encrypt_options = secret_vo.encrypt_options
            trusted_secret_encrypt_options = trusted_secret_vo.encrypt_options

            secret_encrypt_algorithm = secret_encrypt_options.get('encrypt_algorithm')
            trusted_secret_encrypt_algorithm = trusted_secret_encrypt_options.get('encrypt_algorithm')

            if secret_encrypt_algorithm and trusted_secret_encrypt_algorithm and \
                    secret_encrypt_algorithm == trusted_secret_encrypt_algorithm:
                return True
            else:
                raise ERROR_DIFF_SECRET_AND_TRUSTED_SECRET_ENCRYPTED()

        return True



from spaceone.secret.service.secret_service import SecretService
from spaceone.secret.service.secret_group_service import SecretGroupService
from spaceone.secret.service.trusted_secret_service import TrustedSecretService



import logging

from spaceone.core.service import *
from spaceone.core import utils
from spaceone.secret.error.custom import *
from spaceone.secret.manager.secret_group_manager import SecretGroupManager
from spaceone.secret.manager.secret_manager import SecretManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class SecretGroupService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_group_mgr: SecretGroupManager = self.locator.get_manager('SecretGroupManager')

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['name', 'domain_id'])
    def create(self, params):
        """ Create secret group

        Args:
            params (dict): {
                'name': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            secret_group_vo
        """

        return self.secret_group_mgr.create_secret_group(params)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['secret_group_id', 'domain_id'])
    def update(self, params):
        """ Update secret group

        Args:
            params (dict): {
                'secret_group_id' : 'str',
                'name': 'str',
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            secret_group_vo
        """

        secret_group_vo = self.secret_group_mgr.get_secret_group(params['secret_group_id'],
                                                                 params['domain_id'])

        return self.secret_group_mgr.update_secret_group_by_vo(params, secret_group_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['secret_group_id', 'domain_id'])
    def delete(self, params):
        """ Delete secret group

        Args:
            params (dict): {
                'secret_group_id' : 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """

        secret_group_vo = self.secret_group_mgr.get_secret_group(params['secret_group_id'],
                                                                 params['domain_id'])

        self.secret_group_mgr.delete_secret_group_by_vo(secret_group_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['secret_group_id', 'secret_id', 'domain_id'])
    def add_secret(self, params):
        """ Add secret to secret group

        Args:
            params (dict): {
                'secret_group_id' : 'str',
                'secret_id: 'str',
                'domain_id': 'str'
            }

        Returns:
            secret_group_map_vo
        """
        secret_mgr: SecretManager = self.locator.get_manager('SecretManager')
        secret_group_id = params['secret_group_id']
        secret_id = params['secret_id']
        domain_id = params['domain_id']

        secret_group_vo = self.secret_group_mgr.get_secret_group(secret_group_id, domain_id)
        secret_vo = secret_mgr.get_secret(secret_id, domain_id)

        self._check_not_exist_secret_in_group(secret_group_vo, secret_vo)
        secret_group_map_vo = self.secret_group_mgr.add_secret(secret_group_vo, secret_vo)

        return secret_group_map_vo

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['secret_group_id', 'secret_id', 'domain_id'])
    def remove_secret(self, params):
        """ Remove secret from secret group

        Args:
            params (dict): {
                'secret_group_id' : 'str',
                'secret_id: 'str',
                'domain_id': 'str'
            }

        Returns:
            secret_group_map_vo
        """
        secret_mgr: SecretManager = self.locator.get_manager('SecretManager')
        secret_group_id = params['secret_group_id']
        secret_id = params['secret_id']
        domain_id = params['domain_id']

        secret_group_vo = self.secret_group_mgr.get_secret_group(secret_group_id, domain_id)
        secret_vo = secret_mgr.get_secret(secret_id, domain_id)

        secret_group_map_vo = self._get_secret_group_map(secret_group_vo, secret_vo)
        self.secret_group_mgr.remove_secret(secret_group_vo, secret_group_map_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['secret_group_id', 'domain_id'])
    def get(self, params):
        """ Get secret group

        Args:
            params (dict): {
                'secret_group_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            secret_group_vo

        """

        return self.secret_group_mgr.get_secret_group(params['secret_group_id'], params['domain_id'],
                                                      params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['secret_group_id', 'name', 'secret_id', 'domain_id'])
    @append_keyword_filter(['secret_group_id', 'name'])
    def list(self, params):
        """ List secret groups

        Args:
            params (dict): {
                'secret_group_id': 'str',
                'name': 'str',
                'secret_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get('query', {})
        return self.secret_group_mgr.list_secret_groups(query)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['secret_group_id', 'name'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)'
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.secret_group_mgr.stat_secret_groups(query)

    def _check_not_exist_secret_in_group(self, secret_group_vo, secret_vo):
        secret_group_map_vos, total_count = self._list_secret_group_map(secret_group_vo, secret_vo)

        if total_count > 0:
            raise ERROR_ALREADY_EXIST_SECRET_IN_GROUP(secret_id=secret_vo.secret_id,
                                                      secret_group_id=secret_group_vo.secret_group_id)

    def _get_secret_group_map(self, secret_group_vo, secret_vo):
        secret_group_map_vos, total_count = self._list_secret_group_map(secret_group_vo, secret_vo)

        if total_count == 0:
            raise ERROR_NOT_EXIST_SECRET_IN_GROUP(secret_id=secret_vo.secret_id,
                                                  secret_group_id=secret_group_vo.secret_group_id)

        return secret_group_map_vos[0]

    def _list_secret_group_map(self, secret_group_vo, secret_vo):
        query = {
            'filter': [
                {'k': 'secret_group_id', 'v': secret_group_vo.secret_group_id, 'o': 'eq'},
                {'k': 'secret_id', 'v': secret_vo.secret_id, 'o': 'eq'}
            ]
        }

        return self.secret_group_mgr.list_secret_group_maps(query)



from spaceone.core.error import *

class ERROR_DEFINE_SECRET_BACKEND(ERROR_BASE):
    _message = 'Secret Backend is not defined. {backend}'

class ERROR_ALREADY_EXIST_SECRET_IN_GROUP(ERROR_BASE):
    _message = 'Secret is already exist in group.(secret_group_id = {secret_group_id}, secret_id = {secret_id})'


class ERROR_NOT_EXIST_SECRET_IN_GROUP(ERROR_BASE):
    _message = 'Secret is not exist in group.(secret_group_id = {secret_group_id}, secret_id = {secret_id})'


class ERROR_WRONG_ENCRYPT_ALGORITHM(ERROR_HANDLER_CONFIGURATION):
    _message = 'EncryptAlgorithm({encrypt_algorithm}) is not supported.'
    

class ERROR_NOT_EXIST_TRUST_SERVICE_ACCOUNT(ERROR_BASE):
    _message = 'Trust type service account is not exist'


class ERROR_EXIST_RELATED_SECRET(ERROR_BASE):
    _message = 'Related Secret is exist. (secret_id={secret_id})'


class ERROR_DIFF_SECRET_AND_TRUSTED_SECRET_ENCRYPTED(ERROR_BASE):
    _message = 'The encryption algorithm of Secret and Trusted Secret are different.'



import logging

from spaceone.core.manager import BaseManager
from spaceone.secret.model.secret_model import Secret
from spaceone.secret.model.secret_group_model import SecretGroupMap

_LOGGER = logging.getLogger(__name__)


class SecretManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_model: Secret = self.locator.get_model('Secret')

    def create_secret(self, params):
        def _rollback(secret_vo):
            _LOGGER.info(f'[ROLLBACK] Delete secret : {secret_vo.name} ({secret_vo.secret_id})')
            secret_vo.delete()

        secret_vo: Secret = self.secret_model.create(params)

        self.transaction.add_rollback(_rollback, secret_vo)

        return secret_vo

    def update_secret(self, params):
        self.update_secret_by_vo(params, self.get_secret(params['secret_id'], params['domain_id']))

    def update_secret_by_vo(self, params, secret_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["secret_id"]})')
            secret_vo.update(old_data)

        self.transaction.add_rollback(_rollback, secret_vo.to_dict())

        return secret_vo.update(params)

    def delete_secret(self, secret_id, domain_id):
        self.delete_secret_by_vo(self.get_secret(secret_id, domain_id))

    @staticmethod
    def delete_secret_by_vo(secret_vo):
        secret_vo.delete()

    def get_secret(self, secret_id, domain_id, only=None):
        return self.secret_model.get(secret_id=secret_id, domain_id=domain_id, only=only)

    def list_secrets(self, query):
        return self.secret_model.query(**query)

    def stat_secrets(self, query):
        return self.secret_model.stat(**query)

    def get_related_secret_groups(self, secret_vos):
        include_secret_group_filter = {
            'filter': [{
                'k': 'secret_id',
                'v': list(map(lambda secret_vo: secret_vo.secret_id, secret_vos)),
                'o': 'in'
            }]
        }

        secret_group_map_model: SecretGroupMap = self.locator.get_model('SecretGroupMap')
        map_vos, total_count = secret_group_map_model.query(include_secret_group_filter)

        for secret_vo in secret_vos:
            secret_groups = []
            for map_vo in map_vos:
                if map_vo.secret == secret_vo:
                    secret_groups.append(map_vo.secret_group)

            setattr(secret_vo, 'secret_groups', secret_groups)

        return secret_vos



from spaceone.secret.model.secret_model import Secret
from spaceone.secret.model.secret_group_model import SecretGroup, SecretGroupMap
from spaceone.secret.model.trusted_secret_model import TrustedSecret



DATABASE_AUTO_CREATE_INDEX = True
DATABASES = {
    'default': {
        'db': 'secret',
        'host': 'localhost',
        'port': 27017,
        'username': '',
        'password': ''
    }
}

CACHES = {
    'default': {},
    'local': {
        'backend': 'spaceone.core.cache.local_cache.LocalCache',
        'max_size': 128,
        'ttl': 300
    }
}

HANDLERS = {}

BACKEND = ""
CONNECTORS = {
    'SpaceConnector': {
        'backend': 'spaceone.core.connector.space_connector.SpaceConnector',
        'endpoints': {
            'identity': 'grpc://identity:50051'
        }
    },
    'AWSSecretManagerConnector': {},
    'VaultConnector': {
       # 'url': 'http://vault:8200',
       # 'token': 'myroot'
    },
    'ConsulConnector': {
        'host': 'consul',
        'port': 8500
    },
    'EtcdConnector': {
        'host': 'localhost',
        'port': 2379
    },
    'MongoDBConnector': {
        'host': 'localhost',
        'port': 27017,
        'username': '',
        'password': ''
    },
}

ENDPOINTS = {}

LOG = {
    'filters': {
        'masking': {
            'rules': {
                'Secret.create': [
                    'data'
                ],
                'Secret.update_data': [
                    'data'
                ],
                'TrustedSecret.create': [
                    'data'
                ],
                'TrustedSecret.update_data': [
                    'data'
                ]
            }
        }
    }
}



import logging

from spaceone.core.service import *
from spaceone.secret.error.custom import *
from spaceone.secret.manager.identity_manager import IdentityManager
from spaceone.secret.manager.secret_manager import SecretManager
from spaceone.secret.manager.trusted_secret_manager import TrustedSecretManager
from spaceone.secret.manager.secret_connector_manager import SecretConnectorManager

_LOGGER = logging.getLogger(__name__)


@authentication_handler
@authorization_handler
@mutation_handler
@event_handler
class TrustedSecretService(BaseService):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_secret_mgr: TrustedSecretManager = self.locator.get_manager('TrustedSecretManager')

    @transaction(append_meta={
        'authorization.scope': 'DOMAIN'
    })
    @check_required(['name', 'data', 'domain_id'])
    def create(self, params):
        """ Create trusted secret

        Args:
            params (dict): {
                'name': 'str',
                'data': 'dict',
                'tags': 'dict',
                'schema': 'str',
                'service_account_id': 'str',
                'domain_id': 'str
            }

        Returns:
            trusted_secret_vo
        """
        domain_id = params['domain_id']

        if 'service_account_id' in params:
            service_account_info = self._check_service_accounts(params, domain_id)
            params['provider'] = service_account_info.get('provider')

        trusted_secret_vo = self.trusted_secret_mgr.create_trusted_secret(params)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        secret_conn_mgr.create_secret(trusted_secret_vo.trusted_secret_id, params['data'])

        return trusted_secret_vo

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['trusted_secret_id', 'domain_id'])
    def update(self, params):
        """ Update trusted secret

        Args:
            params (dict): {
                'trusted_secret_id': 'str',
                'name': 'str' ,
                'tags': 'dict',
                'domain_id': 'str'
            }

        Returns:
            trusted_secret_vo
        """

        domain_id = params['domain_id']
        trusted_secret_id = params['trusted_secret_id']
        trusted_secret_vo = self.trusted_secret_mgr.get_trusted_secret(trusted_secret_id, domain_id)
        update_trusted_secret_vo = self.trusted_secret_mgr.update_trusted_secret_by_vo(params, trusted_secret_vo)

        return update_trusted_secret_vo

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['trusted_secret_id', 'domain_id'])
    def delete(self, params):
        """ Delete trusted secret

        Args:
            params (dict): {
                'trusted_secret_id': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        secret_mgr: SecretManager = self.locator.get_manager('SecretManager')

        domain_id = params['domain_id']
        trusted_secret_id = params['trusted_secret_id']
        trusted_secret_vo = self.trusted_secret_mgr.get_trusted_secret(trusted_secret_id, domain_id)

        self._check_related_secret(secret_mgr, trusted_secret_id, domain_id)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        secret_conn_mgr.delete_secret(trusted_secret_id)

        self.trusted_secret_mgr.delete_trusted_secret_by_vo(trusted_secret_vo)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['trusted_secret_id', 'data', 'domain_id'])
    def update_data(self, params):
        """ Update trusted secret data through backend Secret service

        Args:
            params (dict): {
                'trusted_secret_id': 'str',
                'data': 'dict',
                'schema': 'str',
                'domain_id': 'str'
            }

        Returns:
            None
        """
        domain_id = params['domain_id']
        trusted_secret_id = params['trusted_secret_id']
        data = params['data']

        trusted_secret_vo = self.trusted_secret_mgr.get_trusted_secret(trusted_secret_id, domain_id)
        self.trusted_secret_mgr.update_trusted_secret_by_vo(params, trusted_secret_vo)

        secret_conn_mgr: SecretConnectorManager = self.locator.get_manager('SecretConnectorManager')
        secret_conn_mgr.update_secret(trusted_secret_id, data)

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['trusted_secret_id', 'domain_id'])
    def get(self, params):
        """ Get trusted secret

        Args:
            params (dict): {
                'trusted_secret_id': 'str',
                'domain_id': 'str',
                'only': 'list'
            }

        Returns:
            trusted_secret_vo
        """

        return self.trusted_secret_mgr.get_trusted_secret(params['trusted_secret_id'],
                                                          params['domain_id'],
                                                          params.get('only'))

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['domain_id'])
    @append_query_filter(['trusted_secret_id', 'name', 'schema', 'provider',
                          'service_account_id', 'domain_id'])
    @append_keyword_filter(['trusted_secret_id', 'name', 'schema', 'provider'])
    def list(self, params):
        """ List trusted secrets

        Args:
            params (dict): {
                'trusted_secret_id': 'str',
                'name': 'str',
                'schema': 'str',
                'provider': 'str',
                'service_account_id': 'str',
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.Query)'
            }

        Returns:
            results (list)
            total_count (int)
        """

        query = params.get('query', {})
        trusted_secret_vos, total_count = self.trusted_secret_mgr.list_trusted_secrets(query)

        return trusted_secret_vos, total_count

    @transaction(append_meta={'authorization.scope': 'DOMAIN'})
    @check_required(['query', 'domain_id'])
    @append_query_filter(['domain_id'])
    @append_keyword_filter(['trusted_secret_id', 'name', 'schema', 'provider'])
    def stat(self, params):
        """
        Args:
            params (dict): {
                'domain_id': 'str',
                'query': 'dict (spaceone.api.core.v1.StatisticsQuery)',
                'user_projects': 'list', // from meta
            }

        Returns:
            values (list) : 'list of statistics data'

        """

        query = params.get('query', {})
        return self.secret_mgr.stat_secrets(query)

    def _check_service_accounts(self, params, domain_id):
        query = {
            'filter': [
                {'k': 'service_account_id', 'v': params['service_account_id'], 'o': 'eq'},
                {'k': 'service_account_type', 'v': 'TRUSTED', 'o': 'eq'},
            ]
        }
        identity_mgr: IdentityManager = self.locator.get_manager('IdentityManager')
        response = identity_mgr.list_service_accounts(query, domain_id)

        _LOGGER.debug(f'[_check_service_accounts] list_service_accounts response: {response}')

        if response.get('total_count', 0) == 0:
            raise ERROR_NOT_EXIST_TRUST_SERVICE_ACCOUNT()
        else:
            return response['results'][0]

    @staticmethod
    def _check_related_secret(secret_mgr, trusted_secret_id, domain_id):
        secret_query = {
            'filter': [
                {'k': 'trusted_secret_id', 'v': trusted_secret_id, 'o': 'eq'},
                {'k': 'domain_id', 'v': domain_id, 'o': 'eq'}
            ]
        }
        secret_vos, secret_total_count = secret_mgr.list_secrets(secret_query)
        if secret_total_count > 0:
            raise ERROR_EXIST_RELATED_SECRET(secret_id=secret_vos[0].secret_id)



from spaceone.secret.info.secret_info import *
from spaceone.secret.info.secret_group_info import *
from spaceone.secret.info.trusted_secret_info import *
from spaceone.secret.info.common_info import *



import logging
from mongoengine import *

from spaceone.core.locator import Locator
from spaceone.core.model.mongo_model import MongoModel

_LOGGER = logging.getLogger(__name__)


class Secret(MongoModel):
    secret_id = StringField(max_length=40, generate_id='secret', unique=True)
    name = StringField(max_length=255, unique_with='domain_id')
    secret_type = StringField(max_length=40, choices=('CREDENTIALS',))
    tags = DictField()
    schema = StringField(max_length=40, null=True, default=None)
    provider = StringField(max_length=40, null=True, default=None)
    encrypted = BooleanField(default=False)
    encrypt_options = DictField()
    service_account_id = StringField(max_length=40, null=True, default=None)
    project_id = StringField(max_length=40, null=True, default=None)
    trusted_secret_id = StringField(max_length=40, null=True, default=None)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'tags',
            'schema',
            'encrypted',
            'encrypt_options',
            'project_id'
        ],
        'minimal_fields': [
            'secret_id',
            'name',
            'secret_type',
            'encrypted'
        ],
        'change_query_keys': {
            'user_projects': 'project_id'
        },
        'ordering': [
            'name'
        ],
        'indexes': [
            # 'secret_id',
            'secret_type',
            'schema',
            'provider',
            'encrypted',
            'service_account_id',
            'project_id',
            'domain_id',
        ]
    }

    @classmethod
    def query(cls, *args, **kwargs):
        change_filter = []

        for condition in kwargs.get('filter', []):
            key = condition.get('k') or condition.get('key')
            if key == 'secret_group_id':
                change_filter.append(cls._change_secret_group_id_filter(condition))

            else:
                change_filter.append(condition)

        kwargs['filter'] = change_filter

        return super().query(*args, **kwargs)

    @staticmethod
    def _change_secret_group_id_filter(condition):
        value = condition.get('v') or condition.get('value')
        operator = condition.get('o') or condition.get('operator')

        map_query = {
            'filter': [{
                'k': 'secret_group_id',
                'v': value,
                'o': operator
            }]
        }
        locator = Locator()
        secret_group_map_model = locator.get_model('SecretGroupMap')

        map_vos, total_count = secret_group_map_model.query(**map_query)

        return {
            'k': 'secret_id',
            'v': list(map(lambda map_vo: map_vo.secret.secret_id, map_vos)),
            'o': 'in'
        }



import logging
from spaceone.core.manager import BaseManager
from spaceone.secret.model.secret_group_model import SecretGroup, SecretGroupMap

_LOGGER = logging.getLogger(__name__)


class SecretGroupManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.secret_group_model: SecretGroup = self.locator.get_model('SecretGroup')
        self.secret_group_map_model: SecretGroupMap = self.locator.get_model('SecretGroupMap')

    def create_secret_group(self, params):
        def _rollback(secret_group_vo):
            _LOGGER.info(
                f'[ROLLBACK] Delete secret group : {secret_group_vo.name} ({secret_group_vo.secret_group_id})')
            secret_group_vo.delete()

        secret_group_vo: SecretGroup = self.secret_group_model.create(params)
        self.transaction.add_rollback(_rollback, secret_group_vo)

        return secret_group_vo

    def update_secret_group(self, params):
        self.update_secret_group_by_vo(params, self.get_secret_group(params['secret_group_id'],
                                                                     params['domain_id']))

    def update_secret_group_by_vo(self, params, secret_group_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["secret_group_id"]})')
            secret_group_vo.update(old_data)

        self.transaction.add_rollback(_rollback, secret_group_vo.to_dict())

        return secret_group_vo.update(params)

    def delete_secret_group(self, secret_group_id, domain_id):
        self.delete_secret_group_by_vo(self.get_secret_group(secret_group_id, domain_id))

    def get_secret_group(self, secret_group_id, domain_id, only=None):
        return self.secret_group_model.get(secret_group_id=secret_group_id, domain_id=domain_id, only=only)

    def list_secret_groups(self, query):
        return self.secret_group_model.query(**query)

    def stat_secret_groups(self, query):
        return self.secret_group_model.stat(**query)

    def list_secret_group_maps(self, query):
        return self.secret_group_map_model.query(**query)

    @staticmethod
    def add_secret(secret_group_vo, secret_vo):
        return secret_group_vo.append('secrets', secret_vo)

    @staticmethod
    def remove_secret(secret_group_vo, secret_group_map_vo):
        secret_group_vo.remove('secrets', secret_group_map_vo)

    @staticmethod
    def delete_secret_group_by_vo(secret_group_vo):
        secret_group_vo.delete()



import logging
from mongoengine import *
from spaceone.core.model.mongo_model import MongoModel

_LOGGER = logging.getLogger(__name__)


class TrustedSecret(MongoModel):
    trusted_secret_id = StringField(max_length=40, generate_id='trusted-secret', unique=True)
    name = StringField(max_length=255, unique_with='domain_id')
    tags = DictField()
    schema = StringField(max_length=40, null=True, default=None)
    provider = StringField(max_length=40, null=True, default=None)
    encrypted = BooleanField(default=False)
    encrypt_options = DictField()
    service_account_id = StringField(max_length=40, null=True, default=None)
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'tags',
            'schema',
            'encrypted',
            'encrypt_options'
        ],
        'minimal_fields': [
            'trusted_secret_id',
            'name',
            'provider',
            'encrypted'
        ],
        'change_query_keys': {},
        'ordering': [
            'name'
        ],
        'indexes': [
            'schema',
            'provider',
            'encrypted',
            'service_account_id',
            'domain_id',
        ]
    }



import logging
import functools
from spaceone.api.secret.v1 import trusted_secret_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.secret.model.trusted_secret_model import TrustedSecret


__all__ = ['TrustedSecretInfo', 'TrustedSecretsInfo']
_LOGGER = logging.getLogger(__name__)


def TrustedSecretInfo(trusted_secret_vo: TrustedSecret, minimal=False):
    info = {
        'trusted_secret_id': trusted_secret_vo.trusted_secret_id,
        'name': trusted_secret_vo.name
    }

    if minimal is False:
        info.update({
            'tags': change_struct_type(trusted_secret_vo.tags),
            'schema': trusted_secret_vo.schema,
            'provider': trusted_secret_vo.provider,
            'service_account_id': trusted_secret_vo.service_account_id,
            'domain_id': trusted_secret_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(trusted_secret_vo.created_at)
        })

    return trusted_secret_pb2.TrustedSecretInfo(**info)


def TrustedSecretsInfo(trusted_secret_vos, total_count, **kwargs):
    results = list(map(functools.partial(TrustedSecretInfo, **kwargs), trusted_secret_vos))
    return trusted_secret_pb2.TrustedSecretsInfo(results=results, total_count=total_count)



import logging
import functools
from spaceone.api.secret.v1 import secret_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.secret.model.secret_model import Secret


__all__ = ['SecretInfo', 'SecretsInfo', 'SecretDataInfo']
_LOGGER = logging.getLogger(__name__)


def SecretDataInfo(secret_data):
    info = {
        'encrypted': secret_data.get('encrypted', False),
        'encrypt_options': change_struct_type(secret_data.get('encrypt_options', {})),
        'data': change_struct_type(secret_data['data'])
    }

    return secret_pb2.SecretDataInfo(**info)


def SecretInfo(secret_vo: Secret, minimal=False):
    info = {
        'secret_id': secret_vo.secret_id,
        'name': secret_vo.name,
        'secret_type': secret_vo.secret_type
    }

    if minimal is False:
        info.update({
            'tags': change_struct_type(secret_vo.tags),
            'schema': secret_vo.schema,
            'provider': secret_vo.provider,
            'service_account_id': secret_vo.service_account_id,
            'trusted_secret_id': secret_vo.trusted_secret_id,
            'project_id': secret_vo.project_id,
            'domain_id': secret_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(secret_vo.created_at)
        })

        if getattr(secret_vo, 'secret_groups', None) is not None:
            secret_group_infos = list(map(lambda secret_group: SecretGroupInfo(secret_group),
                                          secret_vo.secret_groups))

            info.update({
                'secret_groups': change_list_value_type(secret_group_infos)
            })

    return secret_pb2.SecretInfo(**info)


def SecretsInfo(secret_vos, total_count, **kwargs):
    results = list(map(functools.partial(SecretInfo, **kwargs), secret_vos))

    return secret_pb2.SecretsInfo(results=results, total_count=total_count)


def SecretGroupInfo(secret_group_vo):
    return {
        'secret_group_id': secret_group_vo.secret_group_id,
        'name': secret_group_vo.name
    }



from spaceone.api.secret.v1 import secret_group_pb2, secret_group_pb2_grpc
from spaceone.core.pygrpc import BaseAPI


class SecretGroup(BaseAPI, secret_group_pb2_grpc.SecretGroupServicer):

    pb2 = secret_group_pb2
    pb2_grpc = secret_group_pb2_grpc

    def create(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            return self.locator.get_info('SecretGroupInfo', secret_group_service.create(params))

    def update(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            return self.locator.get_info('SecretGroupInfo', secret_group_service.update(params))

    def delete(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            secret_group_service.delete(params)
            return self.locator.get_info('EmptyInfo')

    def add_secret(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            return self.locator.get_info('SecretGroupSecretInfo', secret_group_service.add_secret(params))

    def remove_secret(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            secret_group_service.remove_secret(params)
            return self.locator.get_info('EmptyInfo')

    def get(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            return self.locator.get_info('SecretGroupInfo', secret_group_service.get(params))

    def list(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            secret_group_vos, total_count = secret_group_service.list(params)
            return self.locator.get_info('SecretGroupsInfo', secret_group_vos, total_count,
                                         minimal=self.get_minimal(params))

    def stat(self, request, context):
        params, metadata = self.parse_request(request, context)

        with self.locator.get_service('SecretGroupService', metadata) as secret_group_service:
            return self.locator.get_info('StatisticsInfo', secret_group_service.stat(params))



import logging

from spaceone.core.manager import BaseManager
from spaceone.secret.model.trusted_secret_model import TrustedSecret

_LOGGER = logging.getLogger(__name__)


class TrustedSecretManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.trusted_secret_model: TrustedSecret = self.locator.get_model('TrustedSecret')

    def create_trusted_secret(self, params):
        def _rollback(trusted_secret_vo):
            _LOGGER.info(f'[ROLLBACK] Delete trusted secret : {trusted_secret_vo.name} ({trusted_secret_vo.trusted_secret_id})')
            trusted_secret_vo.delete()

        trusted_secret_vo: TrustedSecret = self.trusted_secret_model.create(params)

        self.transaction.add_rollback(_rollback, trusted_secret_vo)

        return trusted_secret_vo

    def update_trusted_secret(self, params):
        self.update_trusted_secret_by_vo(params, self.get_trusted_secret(params['trusted_secret_id'], params['domain_id']))

    def update_trusted_secret_by_vo(self, params, trusted_secret_vo):
        def _rollback(old_data):
            _LOGGER.info(f'[ROLLBACK] Revert Data : {old_data["name"]} ({old_data["trusted_secret_id"]})')
            trusted_secret_vo.update(old_data)

        self.transaction.add_rollback(_rollback, trusted_secret_vo.to_dict())

        return trusted_secret_vo.update(params)

    def delete_trusted_secret(self, trusted_secret_id, domain_id):
        self.delete_trusted_secret_by_vo(self.get_trusted_secret(trusted_secret_id, domain_id))

    @staticmethod
    def delete_trusted_secret_by_vo(trusted_secret_vo):
        trusted_secret_vo.delete()

    def get_trusted_secret(self, trusted_secret_id, domain_id, only=None):
        return self.trusted_secret_model.get(trusted_secret_id=trusted_secret_id, domain_id=domain_id, only=only)

    def list_trusted_secrets(self, query):
        return self.trusted_secret_model.query(**query)

    def stat_trusted_secrets(self, query):
        return self.trusted_secret_model.stat(**query)



import logging

from google.protobuf.json_format import MessageToDict

from spaceone.core.connector import BaseConnector
from spaceone.core import pygrpc
from spaceone.core.utils import parse_endpoint
from spaceone.core.error import *

__all__ = ['IdentityConnector']
_LOGGER = logging.getLogger(__name__)


class IdentityConnector(BaseConnector):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if 'endpoint' not in self.config:
            raise ERROR_WRONG_CONFIGURATION(key='endpoint')

        if len(self.config['endpoint']) > 1:
            raise ERROR_WRONG_CONFIGURATION(key='too many endpoint')

        for (k, v) in self.config['endpoint'].items():
            e = parse_endpoint(v)
            self.client = pygrpc.client(endpoint=f'{e.get("hostname")}:{e.get("port")}', version=k)

    def get_service_account(self, service_account_id, domain_id):
        response = self.client.ServiceAccount.get({
            'service_account_id': service_account_id,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def list_service_accounts(self, query, domain_id):
        response = self.client.ServiceAccount.list({
            'query': query,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def get_project(self, project_id, domain_id):
        response = self.client.Project.get({
            'project_id': project_id,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    def list_projects(self, query, domain_id):
        response = self.client.Project.list({
            'query': query,
            'domain_id': domain_id
        }, metadata=self.transaction.get_connection_meta())

        return self._change_message(response)

    @staticmethod
    def _change_message(message):
        return MessageToDict(message, preserving_proto_field_name=True)



import logging
from mongoengine import *

from spaceone.core.locator import Locator
from spaceone.core.model.mongo_model import MongoModel
from spaceone.secret.model.secret_model import Secret

_LOGGER = logging.getLogger(__name__)


class SecretGroup(MongoModel):
    secret_group_id = StringField(max_length=40, generate_id='secret-grp', unique=True)
    name = StringField(max_length=255, unique_with='domain_id')
    tags = DictField()
    domain_id = StringField(max_length=255)
    created_at = DateTimeField(auto_now_add=True)

    meta = {
        'updatable_fields': [
            'name',
            'tags'
        ],
        'minimal_fields': [
            'secret_group_id',
            'name'
        ],
        'ordering': [
            'name'
        ],
        'indexes': [
            # 'secret_group_id',
            'domain_id',
        ]
    }

    def append(self, key, data):
        if key == 'secrets':
            secret_group_map_vo = SecretGroupMap.create({
                'secret_group': self,
                'secret': data
            })
            return secret_group_map_vo
        else:
            super().append(key, data)
            return self

    def remove(self, key, data):
        if key == 'secrets':
            data.delete()
        else:
            super().remove(key, data)

    @classmethod
    def query(cls, *args, **kwargs):
        change_filter = []

        for condition in kwargs.get('filter', []):
            key = condition.get('k') or condition.get('key')
            if key == 'secret_id':
                change_filter.append(cls._change_secret_id_filter(condition))

            else:
                change_filter.append(condition)

        kwargs['filter'] = change_filter

        return super().query(*args, **kwargs)

    @staticmethod
    def _change_secret_id_filter(condition):
        value = condition.get('v') or condition.get('value')
        operator = condition.get('o') or condition.get('operator')

        map_query = {
            'filter': [{
                'k': 'secret_id',
                'v': value,
                'o': operator
            }]
        }

        locator = Locator()
        secret_group_map_model = locator.get_model('SecretGroupMap')
        map_vos, total_count = secret_group_map_model.query(**map_query)

        return {
            'k': 'secret_group_id',
            'v': list(map(lambda map_vo: map_vo.secret_group.secret_group_id, map_vos)),
            'o': 'in'
        }


class SecretGroupMap(MongoModel):
    secret_group = ReferenceField('SecretGroup', reverse_delete_rule=CASCADE)
    secret = ReferenceField('Secret', reverse_delete_rule=CASCADE)

    meta = {
        'reference_query_keys': {
            'secret_group': SecretGroup,
            'secret': Secret
        },
        'change_query_keys': {
            'secret_group_id': 'secret_group.secret_group_id',
            'secret_id': 'secret.secret_id'
        },
        'indexes': [
            'secret_group',
            'secret'
        ]
    }
